//**********************
// dsPICer
//
// Condition:
// 7.37MHz Internal RC oscillator, 16x PLL enabled
// Fcy=7.37MHzx16/4=29.48MHz, Tcy=33.92ns
// AN3: A/D input port
// OC2: PWM output port
// Liblary: libp30F2012-coff.a
// Linker: p30f2012.gld
// (C)2007 Y.Onodera
//**********************

#include <p30f2012.h>
#include <uart.h>
#include <timer.h>
#include <outcompare.h>

// configuration
_FWDT(WDT_OFF);
_FGS(CODE_PROT_OFF);
_FOSC(CSW_FSCM_OFF & FRC_PLL16);
_FBORPOR(PBOR_OFF & PWRT_64 & MCLR_EN);

// command definition
#define _Get_version 0x1E
#define _Mic_reset 0x20
#define _dsPIC30F_program_mode 0x0C
#define _dsPIC33F_program_mode 0x0D
#define _dsPIC_puts 0x0F
#define _dsPIC_put 0x11
#define _dsPIC_get 0x12
#define _dsPIC_wait 0x13

// PWM control register
unsigned int _PTCON;
unsigned int _PWMCON1;
unsigned int _PWMCON2;

// pin assaignment
#define CLOCK_PIN		PORTFbits.RF6
#define DATA_PIN		PORTFbits.RF3
#define MCLR_PIN		PORTFbits.RF2
#define VPP_PIN			PORTFbits.RF2
#define VDD_PIN			PORTBbits.RB2
#define TRIS_CLOCK_PIN	TRISFbits.TRISF6
#define TRIS_DATA_PIN	TRISFbits.TRISF3
#define TRIS_MCLR_PIN	TRISFbits.TRISF2
#define TRIS_VDD_PIN	TRISBbits.TRISB2

// global data
char CommandWord;
char DataWordL;
char DataWordH;
char DataWordU;
char Buf[64];

// UART
#define Fcy 7370000*4
#define BaudRate 230400
unsigned int _U1BRG;
unsigned int _U1MODE;
unsigned int _U1STA;
unsigned int TxData;
unsigned int RxData;



// wait 10ms
void wait_10ms(){

	int i;
	int j;

	for(j=0;j<10;j++){
		for(i=0;i<29480/9;i++){
						// Fcy=7.37x16/4=29.48MHz
			asm("nop");		// 29.48MHz = 33.92ns
		}
	}

}


// wait 10us
void wait_10us(){

	int i;
	for(i=0;i<29480/3/2/100;i++){
						// Fcy=7.37x16/4=29.48MHz
		asm("nop");		// 29.48MHz = 33.92ns
	}

}

// wait 1us
void wait_pulse(){

	int i;
	for(i=0;i<29480/3/2/1000;i++){
						// Fcy=7.37x16/4=29.48MHz
		asm("nop");		// 29.48MHz = 33.92ns
		asm("nop");		// 29.48MHz = 33.92ns
		asm("nop");		// 29.48MHz = 33.92ns
		asm("nop");		// 29.48MHz = 33.92ns
	}

}

void DataOut24(){

	int i;

	for(i=0;i<8;i++){
		if( (DataWordL & 1) == 0){
			DATA_PIN = 0;		// L
		}else{
			DATA_PIN = 1;		// H
		}
		wait_pulse();
		CLOCK_PIN = 1;	// H
		wait_pulse();
		CLOCK_PIN = 0;	// L
		DataWordL>>=1;
	}

	for(i=0;i<8;i++){
		if( (DataWordH & 1) == 0){
			DATA_PIN = 0;		// L
		}else{
			DATA_PIN = 1;		// H
		}
		wait_pulse();
		CLOCK_PIN = 1;	// H
		wait_pulse();
		CLOCK_PIN = 0;	// L
		DataWordH>>=1;
	}

	for(i=0;i<8;i++){
		if( (DataWordU & 1) == 0){
			DATA_PIN = 0;		// L
		}else{
			DATA_PIN = 1;		// H
		}
		wait_pulse();
		CLOCK_PIN = 1;	// H
		wait_pulse();
		CLOCK_PIN = 0;	// L
		DataWordU>>=1;
	}
	DATA_PIN = 0;
}


void DataIn16(){

	int i;
	TRIS_DATA_PIN = 1;	// input

	for(i=0;i<8;i++){
		wait_pulse();
		CLOCK_PIN = 1;	// H
		wait_pulse();
		DataWordL>>=1;
		if( DATA_PIN == 0 ){
			DataWordL&=0x7F;
		}else{
			DataWordL|=0x80;
		}
		CLOCK_PIN = 0;	// L
	}

	for(i=0;i<8;i++){
		wait_pulse();
		CLOCK_PIN = 1;	// H
		wait_pulse();
		DataWordH>>=1;
		if( DATA_PIN == 0 ){
			DataWordH&=0x7F;
		}else{
			DataWordH|=0x80;
		}
		CLOCK_PIN = 0;	// L
	}

	TRIS_DATA_PIN=0;	// output
	DATA_PIN = 0;
}


void CommandOut5(){

	int i;
	for(i=0;i<5;i++){
		wait_pulse();
		CLOCK_PIN = 1;	// H
		wait_pulse();
		CLOCK_PIN = 0;	// L
	}

}


void DataOut8(){

	int i;

	for(i=0;i<8;i++){
		if( (DataWordL & 0x80) == 0){
			DATA_PIN = 0;		// L
		}else{
			DATA_PIN = 1;		// H
		}
		wait_pulse();
		CLOCK_PIN = 1;	// H
		wait_pulse();
		CLOCK_PIN = 0;	// L
		DataWordL<<=1;
	}
	DATA_PIN = 0;
}


void CommandOut4(){

	int i;

	for(i=0;i<4;i++){
		if( (CommandWord & 1) == 0){
			DATA_PIN = 0;		// L
		}else{
			DATA_PIN = 1;		// H
		}
		wait_pulse();
		CLOCK_PIN = 1;	// H
		wait_pulse();
		CLOCK_PIN = 0;	// L
		CommandWord>>=1;
	}
	DATA_PIN = 0;

}


void Get_version(){

	WriteUART1(1);
	while(BusyUART1());
	WriteUART1(0);
	while(BusyUART1());

}


void dsPIC30F_program_mode(){

	int i;

	TRIS_CLOCK_PIN = 1;	// input
	TRIS_DATA_PIN = 1;	// input
	TRIS_MCLR_PIN = 0;	// output
	VPP_PIN = 0;		// Vpp=off
	TRIS_VDD_PIN = 0;	// output
	VDD_PIN = 0;		// Vdd=off
	// wait 10ms
	wait_10ms();

	TRIS_CLOCK_PIN = 0;	// output
	CLOCK_PIN = 0;		// L
	TRIS_DATA_PIN = 0;	// output
	DATA_PIN = 0;		// L
	TRIS_VDD_PIN = 0;	// output
	VDD_PIN = 1;		// Vdd=on
	for(i=0;i<8 && VDD_PIN==0;i++){
		VDD_PIN = 1;		// Vdd=on
	}
	// wait 10us
	wait_10us();
	TRIS_MCLR_PIN = 0;	// output
	VPP_PIN = 1;		// Vpp=on
	// wait 25ms
	wait_10ms();
	wait_10ms();
	wait_10ms();

	// For dsPIC30
	CommandWord = 0;
	CommandOut4();
	CommandWord = 0;
	CommandOut4();
	CommandWord = 0;
	CommandOut4();
	CommandWord = 0;
	CommandOut4();

	wait_10us();
	VPP_PIN = 0;		// Vpp=off
	// wait 10us
	wait_10us();
	wait_10us();
	VPP_PIN = 1;		// Vpp=on
	// wait 10ms
	wait_10ms();

	WriteUART1(0);
	while(BusyUART1());

}

void dsPIC33F_program_mode(){

	int i;

	TRIS_CLOCK_PIN = 1;	// input
	TRIS_DATA_PIN = 1;	// input
	TRIS_MCLR_PIN = 0;	// output
	MCLR_PIN = 0;		// Vpp=off
	// wait 10ms
	wait_10ms();

	TRIS_CLOCK_PIN = 0;	// output
	CLOCK_PIN = 0;		// L
	TRIS_DATA_PIN = 0;	// output
	DATA_PIN = 0;		// L
	TRIS_VDD_PIN = 0;	// output
	VDD_PIN = 1;		// Vdd=on
	for(i=0;i<8 && VDD_PIN==0;i++){
		VDD_PIN = 1;		// Vdd=on
	}
	// wait 10ms
	wait_10ms();
	VPP_PIN = 1;		// Vpp=on
	// wait 10us
	wait_10us();
	VPP_PIN = 0;		// Vpp=off
	// wait 10us
	wait_10us();

	// For dsPIC33 Entry Code=4D434851
	DataWordL = 0x4D;
	DataOut8();
	DataWordL = 0x43;
	DataOut8();
	DataWordL = 0x48;
	DataOut8();
	DataWordL = 0x51;
	DataOut8();

	// wait 10us
	wait_10us();
	VPP_PIN = 1;		// Vpp=on
	// wait 25ms
	wait_10ms();
	wait_10ms();
	wait_10ms();
	CommandOut5();

	WriteUART1(0);
	while(BusyUART1());
}


void Mic_reset(){

	TRIS_CLOCK_PIN = 1;	// input
	TRIS_DATA_PIN = 1;	// input
	VPP_PIN = 0;		// Vpp=off
	MCLR_PIN = 0;		// MCLR=L
	wait_10ms();
	TRIS_CLOCK_PIN = 1;	// input
	TRIS_DATA_PIN = 1;	// input
	VPP_PIN = 0;		// Vpp=off
	wait_10ms();
	VDD_PIN = 0;		// Vdd=off
	TRIS_MCLR_PIN = 0;
	TRIS_VDD_PIN = 1;
	WriteUART1(0);
	while(BusyUART1());

}


void dsPIC_put(){

	while(U1STAbits.URXDA==0);	// wait RX data
	DataWordU=ReadUART1();
	while(U1STAbits.URXDA==0);	// wait RX data
	DataWordH=ReadUART1();
	while(U1STAbits.URXDA==0);	// wait RX data
	DataWordL=ReadUART1();

	CommandWord=0;	// SIX
	CommandOut4();
	DataOut24();

	WriteUART1(0);
	while(BusyUART1());

}

void dsPIC_get(){

	CommandWord=1;	// REGOUT
	CommandOut4();
	CommandWord=0;	// NOP
	CommandOut4();
	CommandWord=0;	// NOP
	CommandOut4();
	DataIn16();

	WriteUART1(DataWordH);
	while(BusyUART1());
	WriteUART1(DataWordL);
	while(BusyUART1());

}


void GetPacket(){

	int i;
	for(i=0;i<64;i++){
		while(U1STAbits.URXDA==0);	// wait RX data
		Buf[i]=ReadUART1();
	}

}

void dsPIC_puts(){

	int i;
	int j;
	int k;

	GetPacket();
	while(U1STAbits.URXDA==0);	// wait RX data
	k=ReadUART1();
	j=0;
	for(i=0;i<k;i++){
		DataWordU = Buf[j++];
		DataWordH = Buf[j++];
		DataWordL = Buf[j++];
		CommandWord = 0;	// SIX
		CommandOut4();
		DataOut24();
	}

	WriteUART1(0);
	while(BusyUART1());

}


// wait 2ms
void dsPIC_wait(){

	int i;
	for(i=0;i<29480;i++){
						// Fcy=7.37x16/4=29.48MHz
		asm("nop");		// 29.48MHz = 33.92ns
	}

	WriteUART1(0);
	while(BusyUART1());

}


// Main routine
int main(void)
{
	char i;
	// Use Alternate Interrupt Vector Table
	INTCON2bits.ALTIVT=1;

	// init port
	TRISF = 0x0000;
	TRISB = 0x0000;

	// Trun off UART1
	CloseUART1();
	// Configure UART1 interrupt
	ConfigIntUART1(	UART_RX_INT_DIS &
					UART_TX_INT_DIS);
	// Configure UART1
	// U1BRG=((Fcy/BaudRate)/16)-1
	//      =((117.92MHz/19200)/16)-1
	//      =383
	// BaudRate=Fcy/(16(U1BRG+1))
	//         =117.92MHz/(16(383+1))
	//         =19192.7
	// Error=(19193-19200)/19200
	//      =-0.036%
	_U1BRG=	((Fcy/BaudRate)/16);
	// alternate I/O (U1ATX/U1ARX), N81
	_U1MODE=	UART_EN &
			UART_IDLE_CON &
			UART_ALTRX_ALTTX &
			UART_DIS_WAKE &
			UART_DIS_LOOPBACK &
			UART_EN_ABAUD &
			UART_NO_PAR_8BIT &
			UART_1STOPBIT;
	_U1STA=	UART_INT_TX_BUF_EMPTY &
			UART_TX_PIN_NORMAL &
			UART_TX_ENABLE &
			UART_INT_RX_CHAR &
			UART_ADR_DETECT_DIS &
			UART_RX_OVERRUN_CLEAR;
	// Open UART1
	OpenUART1(_U1MODE, _U1STA, _U1BRG);


	// Confirm to turn off ADC
	ADCON1bits.ADON=0;
	// ^C}[Q܂(PWMLA 1.965MHz = Fcy /15)
	OpenTimer2(T2_ON & T2_GATE_OFF & T2_PS_1_1 & T2_SOURCE_INT, 15-1);
	// PWM ܂
	OpenOC2(OC_IDLE_CON & OC_TIMER2_SRC &
		OC_PWM_FAULT_PIN_DISABLE, 0, 0);
	// Secondary Duty Cycle register(OC1RS)
	// base=15/2=7
	SetDCOC2PWM(7);
	// enable interrupt
	DISICNT = 0x0000;

	// main loop
	while(1)
	{
		while(U1STAbits.URXDA==0);	// wait RX data
		i=ReadUART1();
		switch(i){
			case _Get_version:
				Get_version();
				break;
			case _Mic_reset:
				Mic_reset();
				break;
			case _dsPIC30F_program_mode:
				dsPIC30F_program_mode();
				break;
			case _dsPIC33F_program_mode:
				dsPIC33F_program_mode();
				break;
			case _dsPIC_puts:
				dsPIC_puts();
				break;
			case _dsPIC_put:
				dsPIC_put();
				break;
			case _dsPIC_get:
				dsPIC_get();
				break;
			case _dsPIC_wait:
				dsPIC_wait();
				break;
		}

	}
}


